package com.foo.implementations;



import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.wicket.extensions.markup.html.repeater.util.SortParam;


import com.foo.QueryParam;
import com.foo.domain.Contact;
import com.foo.domain.ContactDao;

/**
 * a dao implementation with an auto-generated embedded database. in a true application this dao
 * would interface with a real database, but because we want to keep dependencies to a minimum we
 * generate our own database here.
 * 
 * @author Igor Vaynberg (ivaynberg)
 * 
 */

//@Repository("contactDao")
public class ContactDaoImpl implements ContactDao
{
	private final Map<Long, Contact> map = Collections.synchronizedMap(new HashMap<Long, Contact>());

	private final List<Contact> fnameIdx = Collections.synchronizedList(new ArrayList<Contact>());

	private final List<Contact> lnameIdx = Collections.synchronizedList(new ArrayList<Contact>());

	private final List<Contact> fnameDescIdx = Collections.synchronizedList(new ArrayList<Contact>());

	private final List<Contact> lnameDescIdx = Collections.synchronizedList(new ArrayList<Contact>());

	/**
	 * Construct.
	 */
	public ContactDaoImpl()
	{
		for (int i = 0; i < 135; i++)
		{
			add(ContactGenerator.getInstance().generate());
		}
		updateIndecies();
	}

	/**
	 * find contact by id
	 * 
	 * @param id
	 * @return contact
	 */
	public Contact get(long id)
	{
		Contact c = map.get(id);
		if (c == null)
			throw new RuntimeException("contact with id [" + id + "] not found in the database");
		return c;
	}

	protected void add(final Contact contact)
	{
		map.put(contact.getId(), contact);
		fnameIdx.add(contact);
		lnameIdx.add(contact);
		fnameDescIdx.add(contact);
		lnameDescIdx.add(contact);
	}

	/**
	 * select contacts and apply sort
	 * 
	 * @param qp
	 *            the query parameter
	 * 
	 * @return list of contacts
	 */
	public Iterator<Contact> find(QueryParam qp)
	{
	
		List<Contact> sublist = getIndex(qp.getSort()).subList(qp.getFirst(),
			qp.getFirst() + qp.getCount());
		return sublist.iterator();
	}

	protected List<Contact> getIndex(SortParam sort)
	{
		if (sort == null)
			return fnameIdx;

		if (sort.getProperty().equals("firstName"))
		{
			return sort.isAscending() ? fnameIdx : fnameDescIdx;
		}
		else if (sort.getProperty().equals("lastName"))
		{
			return sort.isAscending() ? lnameIdx : lnameDescIdx;
		}
		throw new RuntimeException("unknown sort option [" + sort +
			"]. valid fields: [firstName], [lastName]");
	}

	/**
	 * @return number of contacts in the database
	 */
	public int count()
	{
		return fnameIdx.size();
	}

	/**
	 * add contact to the database
	 * 
	 * @param contact
	 */
	public void save(final Contact contact)
	{
		if (contact.getId() == 0)
		{
			contact.setId(ContactGenerator.getInstance().generateId());
			add(contact);
			updateIndecies();
		}
		else
		{
			throw new IllegalArgumentException("contact [" + contact.getFirstName() +
				"] is already persistent");
		}
	}

	/**
	 * delete contact from the database
	 * 
	 * @param contact
	 */
	public void delete(final Contact contact)
	{
		map.remove(contact.getId());

		fnameIdx.remove(contact);
		lnameIdx.remove(contact);
		fnameDescIdx.remove(contact);
		lnameDescIdx.remove(contact);

		contact.setId(0);
	}

	private void updateIndecies()
	{
		Collections.sort(fnameIdx, new Comparator<Contact>()
		{
			public int compare(Contact arg0, Contact arg1)
			{
				return (arg0).getFirstName().compareTo((arg1).getFirstName());
			}
		});

		Collections.sort(lnameIdx, new Comparator<Contact>()
		{
			public int compare(Contact arg0, Contact arg1)
			{
				return (arg0).getLastName().compareTo((arg1).getLastName());
			}
		});

		Collections.sort(fnameDescIdx, new Comparator<Contact>()
		{
			public int compare(Contact arg0, Contact arg1)
			{
				return (arg1).getFirstName().compareTo((arg0).getFirstName());
			}
		});

		Collections.sort(lnameDescIdx, new Comparator<Contact>()
		{
			public int compare(Contact arg0, Contact arg1)
			{
				return (arg1).getLastName().compareTo((arg0).getLastName());
			}
		});

	}

}

